// src/index.ts
import { ExpressiveCodeTheme as ExpressiveCodeTheme2, InlineStyleAnnotation } from "@expressive-code/core";

// src/cache.ts
import { getHighlighter } from "shiki";
import { getStableObjectHash } from "@expressive-code/core";
var highlighterPromise;
var themeCacheKeys = /* @__PURE__ */ new WeakMap();
function getThemeCacheKey(theme) {
  let key = themeCacheKeys.get(theme);
  if (!key) {
    key = `${theme.name}-${getStableObjectHash({ bg: theme.bg, fg: theme.fg, settings: theme.settings })}`;
    themeCacheKeys.set(theme, key);
  }
  return key;
}
async function getCachedHighlighter({ theme, cacheKey }) {
  const themeUsingCacheKey = { ...theme, name: cacheKey };
  if (highlighterPromise === void 0) {
    highlighterPromise = getHighlighter({ theme: themeUsingCacheKey });
  }
  const highlighter = await highlighterPromise;
  if (!highlighter.getLoadedThemes().includes(cacheKey)) {
    await highlighter.loadTheme(themeUsingCacheKey);
  }
  return highlighter;
}

// src/index.ts
import { BUNDLED_THEMES, loadTheme, FontStyle } from "shiki";
async function loadShikiTheme(bundledThemeName) {
  const shikiTheme = await loadTheme(BUNDLED_THEMES.includes(bundledThemeName) ? `themes/${bundledThemeName}.json` : bundledThemeName);
  const shikiThemeWithoutType = { ...shikiTheme };
  delete shikiThemeWithoutType.type;
  return new ExpressiveCodeTheme2(shikiThemeWithoutType);
}
function pluginShiki() {
  return {
    name: "Shiki",
    hooks: {
      performSyntaxAnalysis: async ({ codeBlock, theme }) => {
        const cacheKey = getThemeCacheKey(theme);
        const highlighter = await getCachedHighlighter({ theme, cacheKey });
        const codeLines = codeBlock.getLines();
        let code = codeBlock.code;
        if (isTerminalLanguage(codeBlock.language)) {
          code = code.replace(/<([^>]*[^>\s])>/g, "X$1X");
        }
        const tokenLines = codeBlock.language === "ansi" ? highlighter.ansiToThemedTokens(code, cacheKey) : highlighter.codeToThemedTokens(code, codeBlock.language, cacheKey, { includeExplanation: false });
        tokenLines.forEach((line, lineIndex) => {
          if (codeBlock.language === "ansi")
            removeAnsiSequencesFromCodeLine(codeLines[lineIndex], line);
          let charIndex = 0;
          line.forEach((token) => {
            const tokenLength = token.content.length;
            const tokenEndIndex = charIndex + tokenLength;
            const fontStyle = token.fontStyle || FontStyle.None;
            codeLines[lineIndex].addAnnotation(
              new InlineStyleAnnotation({
                color: token.color || theme.fg,
                italic: (fontStyle & FontStyle.Italic) === FontStyle.Italic,
                bold: (fontStyle & FontStyle.Bold) === FontStyle.Bold,
                underline: (fontStyle & FontStyle.Underline) === FontStyle.Underline,
                inlineRange: {
                  columnStart: charIndex,
                  columnEnd: tokenEndIndex
                }
              })
            );
            charIndex = tokenEndIndex;
          });
        });
      }
    }
  };
}
function isTerminalLanguage(language) {
  return ["shellscript", "shell", "bash", "sh", "zsh"].includes(language);
}
function removeAnsiSequencesFromCodeLine(codeLine, lineTokens) {
  const newLine = lineTokens.map((token) => token.content).join("");
  const rangesToRemove = getRemovedRanges(codeLine.text, newLine);
  for (let index = rangesToRemove.length - 1; index >= 0; index--) {
    const [start, end] = rangesToRemove[index];
    codeLine.editText(start, end, "");
  }
}
function getRemovedRanges(original, edited) {
  const ranges = [];
  let from = -1;
  let orgIdx = 0;
  let edtIdx = 0;
  while (orgIdx < original.length && edtIdx < edited.length) {
    if (original[orgIdx] !== edited[edtIdx]) {
      if (from === -1)
        from = orgIdx;
      orgIdx++;
    } else {
      if (from > -1) {
        ranges.push([from, orgIdx]);
        from = -1;
      }
      orgIdx++;
      edtIdx++;
    }
  }
  if (edtIdx < edited.length)
    throw new Error(`Edited string contains characters not present in original (${JSON.stringify({ original, edited })})`);
  if (orgIdx < original.length)
    ranges.push([orgIdx, original.length]);
  return ranges;
}
export {
  loadShikiTheme,
  pluginShiki
};
//# sourceMappingURL=index.js.map